/*
 * Copyright (C) 2007 OpenedHand Ltd.
 *
 * Author: Jorn Baayen <jorn@openedhand.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef GUPNP_SERVICE_INTROSPECTION_PRIVATE_H
#define GUPNP_SERVICE_INTROSPECTION_PRIVATE_H

#include <libxml/tree.h>

#include "gupnp-service-introspection.h"

G_GNUC_INTERNAL GUPnPServiceIntrospection *
gupnp_service_introspection_new (xmlDoc *scpd);

#endif /* GUPNP_SERVICE_INTROSPECTION_PRIVATE_H */
